@@ -11,24 +11,47 @@ class InvalidFieldName(Exception):
 class NoParameters(Exception):
     pass
 
+def get_order(filter):
+    """
+    Helper function which provides the order query
+    """
+    order = ""
+    for key, value in filter.items():
+        if key == "order_by":
+            if value.startswith("-"):
+                value = value[1:]
+                order = "ORDER BY %s " % (value) + "DESC"
+            else:
+                order = "ORDER BY %s " % (value) + "ASC"
+    return order
+
+
+def get_objects(cls, filter):
+    """
+    Helper function to return the fetched values in the right order
+    """
+    objects = []
+    mapped = {}
+    for item in cls.adaptor.get_by_filter(filter, cls.tablename, cls.fields.keys()):
+        object = cls(*item)
+        mapped = zip(cls.fields.keys(), item)
+        for item in mapped:
+            setattr(object, item[0], item[1])
+        objects.append(object)
+    return objects
+
+
 class DbObject(object):
     """
-    The main model which provides the ORM
+    The class from which the classes in the application inherits
     """
     def __init__(self):
         self.adaptor = None
 
     @classmethod
-    def all(cls):
-        return [cls(*item) for item in cls.adaptor.get_all(cls.tablename,
-                                                           cls.fieldnames)]
-
-    @classmethod
-    def ordered(cls, criteria=None, direction='ASC'):
-        return [cls(*item) for item in cls.adaptor.get_ordered(criteria,
-                                                               direction,
-                                                               cls.tablename,
-                                                               cls.fieldnames)]
+    def all(cls, **kwargs):
+        filter = "1=1 " + get_order(kwargs)
+        return get_objects(cls, filter)
 
     @classmethod
     def get(cls, *args, **kwargs):
@@ -38,45 +61,103 @@ class DbObject(object):
 
     @classmethod
     def filter(cls, *args, **kwargs):
-
         if not kwargs:
             raise NoParameters('No parameters provided')
-
         filters = []
+        order = ""
 
         for key, value in kwargs.iteritems():
-            filters.append("%s='%s'" % (key, value))
-        filter = (' and ').join(filters)
+            if key.split('__')[0] == "max":
+                key_value = cls.adaptor.get_key_value(key.split('__')[1], value, cls.tablename, cls.fields)
 
-        return [cls(*item) for item in cls.adaptor.get_by_filter(
-                                                                filter,
-                                                                cls.tablename,
-                                                                cls.fieldnames)]
+                filters.append("%s<=%s" % key_value)
+            elif key.split('__')[0] == "min":
+                key_value = cls.adaptor.get_key_value(key.split('__')[1], value, cls.tablename, cls.fields)
+
+                filters.append("%s>=%s" % key_value)
+            elif key == "order_by":
+                continue
+            else:
+                key_value = cls.adaptor.get_key_value(key, value, cls.tablename, cls.fields)
+                filters.append("%s=%s" % key_value)
+        filter = (" AND ").join(filters)
+        if not filter:
+            filter = "1=1"
+        filter += " " + get_order(kwargs)
+        return get_objects(cls, filter)
 
     @classmethod
+    """
+    This method can be called to create an object
+
+    :param class: The class which calls this method
+    :return: The freshly created object
+    """
     def create(cls, *args, **kwargs):
-        return cls.adaptor.create(cls, cls.tablename, cls.fieldnames, kwargs)
+        return cls.adaptor.create(cls, cls.tablename, cls.fields.keys(), kwargs)
 
     def save(self, *args, **kwargs):
+        """
+        Function can be called to create or update an object
+        """
         if self.id:
-            self.update(self.tablename, self.fieldnames, kwargs)
+            self.update(self.tablename, self.fields.keys(), kwargs)
         else:
-            for fieldname in self.fieldnames:
+            for fieldname in self.fields.keys():
                 kwargs[fieldname] = getattr(self, fieldname)
-            self.create(self, self.tablename, self.fieldnames, **kwargs)
+            self.create(self, self.tablename, self.fields.keys(), **kwargs)
 
     def update(self, *args, **kwargs):
-        return self.adaptor.update(self.tablename, self.fieldnames, self)
+        return self.adaptor.update(self.tablename, self.fields.keys(), self)
 
     def delete(self, *args, **kwargs):
         return self.adaptor.delete(self.tablename, self)
 
     @classmethod
+    """
+    Function which can be used to select items from the database by pure SQL
+
+    :param query: A pure SQL select query
+    :return: An object of the calling class
+    """
     def pure(cls, *args):
         return [cls(*item) for item in cls.adaptor.get_by_sql(args[0])]
 
+    @classmethod
+    def raw(cls, query, *args):
+        """
+        Executes a query
+        """
+        con = cls.adaptor.get_connection()
+        cur = con.cursor()
+        cur.execute(query)
+        try:
+            fetch = cur.fetchall()
+        except:
+            fetch = None
+        con.commit()
+        return fetch
+
+    @classmethod
+    def create_table(cls):
+        """
+        Helper function to create tables based on their fieldnames and tablename
+        """
+        return cls.adaptor.create_table(cls)
+
 
 class Adaptor():
+    """
+    Class to provide some common methods which can be used by more than one
+    child adaptor
+    """
+
+    def get_key_value(self, key, value, *args):
+        tablename = args[0]
+        fields = args[1]
+        if fields[key] == 'char':
+            value = "'%s'" % value
+        return (key, value)
 
     def update(self, tablename, fieldnames, *args):
         _object = args[0]
@@ -114,6 +195,7 @@ class PostgreSQLAdaptor(Adaptor):
 
     def get_connection(self):
         import psycopg2
+        psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
         con = psycopg2.connect(host=self.host,
                               database=self.database,
                               user=self.username,
@@ -133,12 +215,6 @@ class PostgreSQLAdaptor(Adaptor):
                                                 filter)
         return self.get_by_sql(query, *args)
 
-    def get_all(cls, *args):
-        return cls.get_by_filter("1 = 1", *args)
-
-    def get_ordered(cls, criteria, direction, *args):
-        return cls.get_by_filter("1 = 1 ORDER BY %s %s" % (criteria, direction), *args)
-
     def create(self, cls, tablename, fieldnames, *args, **kwargs):
         empty_fields = set(fieldnames)
         field_values = {}
@@ -170,6 +246,26 @@ class PostgreSQLAdaptor(Adaptor):
         con.commit()
         return cls.get(id=_id[0])
 
+    def create_table(self, cls):
+        fields = ""
+        for key, value in cls.fields.items():
+            if key == 'id':
+                fields += "id SERIAL PRIMARY KEY, "
+            if value == 'char':
+                fields += "%s VARCHAR(200) DEFAULT NULL, " % key
+            if value == 'int' and not key == 'id':
+                fields += "%s INT DEFAULT 0, " % key
+            if value == 'text':
+                fields += "%s TEXT DEFAULT NULL, " % key
+            if value == 'bool':
+                fields += "%s BOOLEAN, " % key
+            if value == 'date':
+                fields += "%s TIMESTAMP DEFAULT NULL, " % key
+        query = """CREATE TABLE IF NOT EXISTS %s (%s)""" % (cls.tablename,
+                                                              fields[:-2])
+        cls.raw(query)
+        return True
+
 
 class MySQLAdaptor(Adaptor):
 
@@ -184,7 +280,9 @@ class MySQLAdaptor(Adaptor):
         con = MySQLdb.connect(host=self.host,
                               db=self.database,
                               user=self.username,
-                              passwd=self.password)
+                              passwd=self.password,
+                              use_unicode=True,
+                              charset='utf8')
         return con
 
     def get_by_sql(self, query, *args):
@@ -200,12 +298,6 @@ class MySQLAdaptor(Adaptor):
                                                 filter)
         return self.get_by_sql(query, *args)
 
-    def get_all(cls, *args):
-        return cls.get_by_filter("1 = 1", *args)
-
-    def get_ordered(cls, criteria, direction, *args):
-        return cls.get_by_filter("1 = 1 ORDER BY %s %s" % (criteria, direction), *args)
-
     def get_last_inserted_id(self, con):
         cur = con.cursor()
         cur.execute("""SELECT last_insert_id()""")
@@ -235,6 +327,29 @@ class MySQLAdaptor(Adaptor):
         con.commit()
         return cls.get(id=self.get_last_inserted_id(con))
 
+    def create_table(self, cls):
+        fields = ""
+        pk = ""
+        for key, value in cls.fields.items():
+            if key == 'id':
+                fields += "id INT NOT NULL AUTO_INCREMENT, "
+                pk = ", PRIMARY KEY (id)"
+            if value == 'char':
+                fields += "%s VARCHAR(200) DEFAULT NULL, " % key
+            if value == 'int' and not key == 'id':
+                fields += "%s INT DEFAULT 0, " % key
+            if value == 'text':
+                fields += "%s TEXT DEFAULT NULL, " % key
+            if value == 'bool':
+                fields += "%s BOOLEAN, " % key
+            if value == 'date':
+                fields += "%s DATETIME DEFAULT NULL, " % key
+        query = """CREATE TABLE IF NOT EXISTS %s (%s %s)""" % (cls.tablename,
+                                                              fields[:-2],
+                                                              pk)
+        cls.raw(query)
+        return True
+
 
 class SQLiteAdaptor(Adaptor):
 
@@ -246,6 +361,18 @@ class SQLiteAdaptor(Adaptor):
         con = sqlite3.connect(self.database)
         return con
 
+    def get_key_value(self, key, value, *args):
+        tablename = args[0]
+        fields = args[1]
+        if fields[key] == 'bool':
+            if value:
+                value = 1
+            else:
+                value = 0
+        if fields[key] == 'char':
+            value = "'%s'" % value
+        return (key, value)
+
     def get_by_sql(self, query, *args):
         cur = self.get_connection().cursor()
         cur.execute(query)
@@ -259,12 +386,6 @@ class SQLiteAdaptor(Adaptor):
                                                 filter)
         return self.get_by_sql(query, *args)
 
-    def get_all(cls, *args):
-        return cls.get_by_filter("1 = 1", *args)
-
-    def get_ordered(cls, criteria, direction, *args):
-        return cls.get_by_filter("1 = 1 ORDER BY %s %s" % (criteria, direction), *args)
-
     def get_last_inserted_id(self, con):
         cur = con.cursor()
         cur.execute("""SELECT last_insert_rowid()""")
@@ -323,80 +444,22 @@ class SQLiteAdaptor(Adaptor):
         con.cursor().execute(query, [_object.id])
         con.commit()
 
+    def create_table(self, cls):
+        fields = ""
+        for key, value in cls.fields.items():
+            if key == 'id':
+                fields += "id INTEGER PRIMARY KEY AUTOINCREMENT, "
+            if value == 'char':
+                fields += "%s VARCHAR(200) DEFAULT NULL, " % key
+            if value == 'int' and not key == 'id':
+                fields += "%s INTEGER DEFAULT 0, " % key
+            if value == 'text':
+                fields += "%s TEXT DEFAULT NULL, " % key
+            if value == 'bool':
+                fields += "%s BOOLEAN, " % key
+            if value == 'date':
+                fields += "%s TIMESTAMP DEFAULT NULL, " % key
 
-class MyDbAdaptor(Adaptor):
-
-    import db_driver
-    def __init__(self, database):
-        self.database = database
-
-    def get_connection(self):
-        con = db_driver.connect(self.database)
-        return con
-
-    def get_by_sql(self, query, *args):
-        con = self.get_connection()
-        return con.execute(query)
-
-    def get_by_filter(self, filter, *args):
-        tablename = args[0]
-        fieldnames = args[1]
-        query = "SELECT %s FROM %s WHERE %s" % (', '.join(fieldnames),
-                                                tablename,
-                                                filter)
-        return self.get_by_sql(query, *args)
-
-    def get_ordered(cls, criteria, direction, *args):
-        return cls.get_by_filter("1=1 ORDER BY %s %s" % (criteria, direction), *args)
-
-    def get_all(cls, *args):
-        return cls.get_by_filter("1=1", *args)
-
-    def update(self, tablename, fieldnames, *args):
-        _object = args[0]
-        settings = ""
-        for fieldname in fieldnames:
-            settings += ", " + fieldname + "=%s"
-        settings = settings[2:]
-
-        query = "UPDATE %s SET %s WHERE id=%%s" % (tablename, settings)
-
-        values = []
-        for fieldname in fieldnames:
-            values.append(getattr(_object, fieldname))
-        values.append(_object.id)
-
-        con = self.get_connection()
-        con.execute(query % tuple(values))
-
-    def create(self, cls, tablename, fieldnames, *args, **kwargs):
-        empty_fields = set(fieldnames)
-        field_values = {}
-
-        for key, value in args[0].iteritems():
-            if key not in fieldnames:
-                raise InvalidFieldName
-            else:
-                empty_fields -= set([key])
-                field_values[key] = value
-
-        for key in empty_fields:
-            field_values[key] = None
-
-        query = "INSERT INTO %s (%s) VALUES (%s)" % (
-                                       tablename,
-                                       ', '.join(field_values.keys()),
-                                       ', '.join(['%s'] * len(fieldnames)))
-
-        values = []
-        for value in field_values.values():
-            values.append(value)
-
-        con = self.get_connection()
-        return cls.get(id=con.execute(query % tuple(values)))
-
-    def delete(self, tablename, *args):
-        _object = args[0]
-        query = "DELETE FROM %s WHERE id=%s" % (tablename, _object.id)
-        con = self.get_connection()
-        con.execute(query)
\ No newline at end of file
+        query = """CREATE TABLE IF NOT EXISTS %s (%s)""" % (cls.tablename, fields[:-2])
+        cls.raw(query)
+        return True
\ No newline at end of file
